#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Solicitor: uses metadata.json to determine input parameters for ingestion
"""
import json
import pathlib
from typing import List

import requests


class Solicitor:
    def __init__(self, filename: str, arg_type: str, urls: List):
        self.filename = filename
        self.argument = arg_type
        self.capability_url = urls[0]
        self.workflow_url = urls[1]
        self.metadata = self.solicit_contents()

    def solicit_contents(self) -> dict:
        with open(self.filename) as file:
            metadata = json.loads(file.read())
            return metadata

    def solicit_workflow_directory_name(self):
        destination_path = pathlib.Path(self.metadata["destinationDirectory"])
        return destination_path.stem

    def solicit_initial_directory_name(self, wf_id: int):
        current_execution_url = f"{self.capability_url}/capability/request/execution/{wf_id}"
        current_execution = requests.get(current_execution_url)

        initial_execution_url = f"{self.capability_url}/capability/request/{current_execution.json()['capability_request_id']}/version/1/execution"
        initial_execution = requests.get(initial_execution_url)

        # this part is correct.... How to get here
        wf_request_url = f"{self.workflow_url}/workflows/std_calibration/requests/{initial_execution.json()['current_workflow_request_id']}"
        wf_request = requests.get(wf_request_url)

        initial_path = pathlib.Path(wf_request.json()["results_dir"])
        return initial_path.stem

    def solicit_processing_creation_time(self):
        # incoming datetime: 2021-07-06T21:50:48
        # transformation for NGAS friendliness: 2021_07_06T21_50_48
        creation_time = self.metadata["creationTime"].replace("-", "_").replace(":", "_")
        return creation_time

    def solicit_parameters(self):
        metadata = self.solicit_contents()
        project_info = metadata["projectMetadata"]
        parameters = {
            "sdmId": metadata["fileSetIds"],
            "telescope": project_info["telescope"],
            "workflowName": metadata["workflowName"],
            "workflowRequestId": metadata["systemId"],
            "spl": metadata["productLocator"],
            "processingStart": self.solicit_processing_creation_time(),
            "destinationDir": metadata["destinationDirectory"],
            "workflowDir": self.solicit_workflow_directory_name(),
            "initialVersionDir": self.solicit_initial_directory_name(metadata["systemId"]),
        }
        if parameters["workflowDir"] == parameters["initialVersionDir"]:
            parameters["multiVersion"] = False
        else:
            parameters["multiVersion"] = True

        if self.argument == "image":
            parameters["calSpl"] = metadata["calProductLocator"]

        return parameters
