/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import { Component, Input, OnInit } from "@angular/core";
import { ModalDismissReasons, NgbModal } from "@ng-bootstrap/ng-bootstrap";
import { CapabilityRequest } from "../../../../model/capability-request";
import { CapabilityRequestService } from "../../../../services/capability-request.service";
import { CapabilityVersion } from "../../../../model/capability-version";

@Component({
  selector: "app-create-new-version-form",
  templateUrl: "./create-new-version-form.component.html",
  styleUrls: ["./create-new-version-form.component.scss"],
})
export class CreateNewVersionFormComponent implements OnInit {
  closeResult = "";
  @Input() capabilityRequest: CapabilityRequest;
  parametersString: string;
  inputFileList: FileList;

  constructor(
    private modalService: NgbModal,
    private capabilityRequestService: CapabilityRequestService,
  ) {}

  /**
   * Open and display modal and process method by which the modal is closed
   * @param content  Content of modal; in this case, parameters and files field
   */
  public open(content) {
    this.modalService.open(content, { ariaLabelledBy: "modal-title", centered: true }).result.then(
      (result) => {
        if (result === "create") {
          // Create button clicked; create new version from given settings
          this.createNewVersion();
        } else {
          // Form was exited by clicking out of it or pressing ESC
          this.resetForm();
        }
      },
      (reason) => {
        // Form was exited using Cancel button or X
        this.resetForm();
      },
    );
  }

  /**
   * Reset any form values set
   * @private
   */
  private resetForm() {
    this.parametersString = this.getStringifiedParameters();
    this.inputFileList = undefined;
  }

  /**
   * Get string version of capability request parameters
   * @return string  Stringified parameters
   */
  public getStringifiedParameters(): string {
    return JSON.stringify(this.capabilityRequest.parameters);
  }

  /**
   * Set files to be uploaded to variable
   * @param inputFileList  List of files from modal form
   */
  public setInputFileList(inputFileList: FileList) {
    this.inputFileList = inputFileList;
  }

  /**
   * Operation performed when the modal is submitted; passes form results to capability request service
   * @private
   */
  private createNewVersion() {
    // Make REST call to create new version with entered parameters and files
    const idNumber = parseInt(this.capabilityRequest.id);
    const createNewVersionObservable = {
      next: (response) => {
        if (this.inputFileList !== undefined) {
          this.capabilityRequestService.addFilesToVersion(
            this.capabilityRequest.id,
            this.inputFileList,
          );
        }
      },
      error: (error) => alert("Error when creating new version:" + error),
    };
    this.capabilityRequestService
      .createNewVersion(idNumber, this.parametersString, this.inputFileList)
      .subscribe(createNewVersionObservable);
  }

  ngOnInit() {
    this.parametersString = this.getStringifiedParameters();
  }
}
