#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

""" Classes for notification"""
import json
import logging
import smtplib
import subprocess

import requests
from pycapo import CapoConfig

from workspaces.capability.schema import CapabilityRequest
from workspaces.notification.services.interfaces import (
    NotificationInfoIF,
    NotificationServiceIF,
)

# pylint: disable=C0103, E0401, R0903, W1201

logger = logging.getLogger(__name__)


class NotificationServiceRESTClient(NotificationServiceIF):
    def __init__(self):
        self.url = CapoConfig().settings("edu.nrao.workspaces.NotificationSettings").serviceUrl
        self.linkUrl = CapoConfig().settings("edu.nrao.workspaces.UISettings").serviceUrl
        self.sendPI = CapoConfig().getboolean("edu.nrao.workspaces.NotificationSettings.sendPIEmail")

    def notify_submitted(self, request: CapabilityRequest):
        # bail out early if we have no user email
        if "user_email" not in request.current_version.parameters:
            logger.info("Not sending notification because no user email supplied")
            return

        user_email = request.current_version.parameters["user_email"]
        requests.post(
            f"{self.url}/notify/submitted_email/send",
            json={
                "destination_email": user_email,
                "request_id": request.id,
                "capability_name": request.capability.name,
                "status_link": self.linkUrl + "/request-status/" + str(request.id),
            },
        )

    def notify_complete(self, request: CapabilityRequest):
        # bail out early if we have no user email
        if "user_email" not in request.current_version.parameters:
            logger.info("Not sending notification because no user email supplied")
            return

        user_email = request.current_version.parameters["user_email"]
        requests.post(
            f"{self.url}/notify/complete_email/send",
            json={
                "destination_email": user_email,
                "request_id": request.id,
                "capability_name": request.capability.name,
            },
        )

    def notify_failed(self, request: CapabilityRequest):
        if "user_email" not in request.current_version.parameters:
            logger.info("Not sending notification because no user email supplied")
            return

        user_email = request.current_version.parameters["user_email"]
        requests.post(
            f"{self.url}/notify/failed_email/send",
            json={
                "destination_email": user_email,
                "request_id": request.id,
                "capability_name": request.capability.name,
            },
        )

    def notify_ingestion_complete(self, request: CapabilityRequest):
        """
        Notify the user that their request has been successfully ingested into the archive

        :param request: Capability request that the notification is regarding
        """

        logger.info(f"Wresting contact info for PI & CoIs of request #{request.id}")
        # TODO:find way to make sure we're using the correct version's parameters
        with subprocess.Popen(
            ["aat_wrest", "-sc", request.current_version.parameters["product_locator"]],
            stdout=subprocess.PIPE,
        ) as project_data:
            data = project_data.communicate()[0]
            project = json.loads(data)
            project_code = project["projectCode"]

            with subprocess.Popen(
                ["contacts_wrest", project_code],
                stdout=subprocess.PIPE,
            ) as wrested_contacts:
                contacts = wrested_contacts.communicate()[0]
                if "ERROR" in contacts.decode():
                    logger.info(f"Not sending notification because contact wrester failed for project {project_code}")
                    return

        email_list = json.loads(contacts.decode())

        if self.sendPI:
            if isinstance(email_list, list):
                send_to = ", ".join(email_list)
            else:
                send_to = email_list
        else:
            analyst_email = CapoConfig().settings("edu.nrao.workspaces.NotificationSettings").analystEmail

            if isinstance(email_list, list):
                faked_emails = []

                while len(faked_emails) != len(email_list):
                    faked_emails.append(analyst_email)

                if len(faked_emails) == 0:
                    faked_emails.append(analyst_email)

                send_to = ", ".join(faked_emails)

            else:
                send_to = analyst_email

        if request.current_version.workflow_metadata:
            if "qa" in request.current_version.workflow_metadata:
                notes = request.current_version.workflow_metadata["qa"]
            else:
                notes = "No QA notes found."
        else:
            notes = "No QA notes found."

        requests.post(
            f"{self.url}/notify/ingested_email/send",
            json={
                "destination_email": send_to,
                "request_id": request.id,
                "capability_name": request.capability.name,
                "qa": notes,
            },
        )

    def notify_ingestion_failed(self, request: CapabilityRequest):
        """
        Notify the user that their request has failed to be ingested into the archive

        :param request: Capability request that the notification is regarding
        """
        analyst_email = CapoConfig().settings("edu.nrao.workspaces.NotificationSettings").analystEmail
        requests.post(
            f"{self.url}/notify/ingestion_failed_email/send",
            json={
                "destination_email": analyst_email,
                "request_id": request.id,
                "capability_name": request.capability.name,
            },
        )

    def notify_qa_ready(self, request: CapabilityRequest):
        """
        Notify the appropriate parties that a request is ready for QA

        :param request: Capability request
        """
        logger.info("SENDING QA-READY NOTIFICATION!!!")
        analyst_email = CapoConfig().settings("edu.nrao.workspaces.NotificationSettings").analystEmail
        requests.post(
            f"{self.url}/notify/ready-for-QA_email/send",
            json={
                "destination_email": analyst_email,
                "request_id": request.id,
                "capability_name": request.capability.name,
                "status_link": self.linkUrl + "/request-status/" + str(request.id),
            },
        )


class NotificationService:
    def __init__(self, info: NotificationInfoIF):
        self.info = info

    def send_email(self, template_name, parameters: dict):
        smtp_server = "smtp.aoc.nrao.edu"
        sender_email = "do-not-reply@nrao.edu"
        receiver_email = parameters["destination_email"]

        template = self.info.lookup_template(template_name)

        if template is None:
            logger.info("No template '" + template_name + "' found.")

        file = template.render(parameters)

        with smtplib.SMTP(smtp_server) as server:
            if isinstance(receiver_email, list):
                if not receiver_email:
                    return "No PI email found."
                for contact in receiver_email:
                    r = server.sendmail(sender_email, contact, file.content)
            else:
                r = server.sendmail(sender_email, receiver_email, file.content)

        return r
